@@ -18,9 +18,11 @@ _launcher_backend_modules = [
 _launchers = {}
 _scheduler_backend_modules = [
     'reframe.core.schedulers.local',
-    'reframe.core.schedulers.slurm',
+    'reframe.core.schedulers.lsf',
     'reframe.core.schedulers.pbs',
-    'reframe.core.schedulers.sge'
+    'reframe.core.schedulers.oar',
+    'reframe.core.schedulers.sge',
+    'reframe.core.schedulers.slurm'
 ]
 _schedulers = {}
 
